% Compute the element load vectors corresponding to prescribed pressure
% flux. (Pressure flux boundary condition is stated as
%      rho*a_n=-(d p/d n)
% where a_n= prescribed acceleration, n=direction along the outer normal
%
% function evs = pressure_flux_loads (self, geom, P, fi)
%
% Return an array of the element loads so they may be assembled.
%     
% Arguments
%     geom=geometry field
%     P=pressure field
%     center=3-D point with respect to which the torque is taken
%
function [F,T] = pressure_resultant (self, geom, P, center)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells(1),'nfens');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Precompute basis f. values + basis f. gradients wrt parametric coor
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Prepare some data: connectivity and the element matrices
    conns = get(gcells, 'conn'); % connectivity
    xs =get(geom,'values');% Retrieve Geometry information
    F=zeros(get(geom,'dim'),1);
    T=zeros(get(geom,'dim'),1);
    for i=1:size(conns,1)
        conn =conns(i,:);
        x=xs(conn,:);
        nP = gather(P, conn, 'values');
        for j=1:npts_per_gcell
            Y=Ns{j}'*x; % Point at which pressure force acts
            J = Jacobian_matrix(gcells,Nders{j},x);
            Jac = Jacobian_surface(gcells,conn, Ns{j}, J, x);
            n= normal(self,Y,J);
            dF =(nP'*Ns{j})*n;
            F = F + (Jac*w(j)*dF);
            dT =cross((Y-center)',dF);
            T = T + (Jac*w(j)*dT);
        end
    end
end
