% Compute the element load vectors.
%
% function evs = distrib_body_loads_global(self, geom0, u1, Rfield1, dchi, fi)
%
%     geom0=initial geometry field; locations of nodes
%     u1=current geometry increment field; displacements of nodes
%     Rfield1=field of nodal frames, current deformation
%     dchi=field of translation and rotation increments
%             Equation numbers are taken from the field dchi.  
%     fi=force intensity object, must be uniform across each element;
%           the force intensity is given in the global coordinates.
%
% Compute the element load vectors corresponding to applied distributed
% load. Here it means force per unit volume of the beam, 
% in the configuration u1,Rfield1.
% 
% Note: the force intensity must be uniform across the entire element.
% Note: the force intensity is given in the global coordinates.
% 
% Return an array of the load vectors so they may be assembled.
% 
function evs = distrib_body_loads_global(self, geom0, u1, Rfield1, dchi, fi)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells,'nfens');
    dim = get(geom0,'dim');
    % Pre-allocate the element matrices
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(dchi,conns(i,:),'eqnums');
        Fe{i} =zeros(get(geom0,'dim')*nfens,1);
    end
    Xs =gather(geom0,(1:get(geom0,'nfens')),'values','noreshape');
    U1s =gather(u1,(1:get(u1,'nfens')),'values','noreshape');
    Rs = gather(Rfield1, (1:get(geom0,'nfens')), 'values', 'noreshape'); % displacement in step n
    X1s=Xs+U1s;
    % Material
    mat = get(self.feblock, 'mater');
    prop=get(mat, 'property' );
    E=get(prop , 'E' );% Young's modulus
    G=get(prop , 'G' );% shear modulus
    %     matstates = get(self.feblock, 'matstates');
    % Section properties
    A=get(gcells,'A');
    I1=get(gcells,'I1');
    I2=get(gcells,'I2');
    I3=get(gcells,'I3');
    J=get(gcells,'J');
    x1x2_vector=get(gcells,'x1x2_vector');
    % Now loop over all gcells in the block
    Z=zeros(3,3);
    for i=1:ngcells
        conn = conns(i,:); % connectivity
        x0 = Xs(conn,:); % initial coordinates of nodes
        x1 = X1s(conn,1:3); % current coordinates of nodes
        RI =reshape(Rs(conn(1),:),3,3);% current directors at node 1
        RJ =reshape(Rs(conn(2),:),3,3);% current directors at node 2
        [L1,T,d_N]=local_frames(x0,x1x2_vector(:,i),x1,RI,RJ);
        Te=[T Z Z Z; ...
            Z T Z Z;...
            Z Z T Z;...
            Z Z Z T];
        f=A(i)*get_magn(fi,0);% Convert to force per unit length
        fL =T'*f; % local distributed load components
        t = [fL(1)*L1/2;
            fL(2)*L1/2;
            fL(3)*L1/2;
            0;
            -fL(3)*L1^2/12;
            +fL(2)*L1^2/12;
            fL(1)*L1/2;
            fL(2)*L1/2;
            fL(3)*L1/2;
            0;
            +fL(3)*L1^2/12;
            -fL(2)*L1^2/12;
            ];
        Fe{i} = Te*t;
    end
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
end
