% Compute the element load vectors corresponding to applied body load.
%
% function evs = body_loads(self, geom, w, bl)
%
% Return an array of them so they may be assembled.
%    Call as
% evs = body_loads(feb, geom, u, bl)
%     geom=geometry field
%     w=displacement field
%     bl= body load
%
function evs = body_loads(self, geom, w, bl)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells,'nfens');
    dim = get(geom,'dim');
    if dim ~= 1
        error(['wrong dimension: need dim=1']);
    end
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    weights = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Get ready some variables
    conns = get(gcells, 'conn'); % connectivity
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(w,conns(i,:),'eqnums');
        Fe{i} =zeros(dim*nfens,1);
    end
    % Now loop over all gcells in the block
    for i=1:size(conns,1)
        conn = conns(i,:); % connectivity
        x = gather(geom, conn, 'values', 'noreshape'); % coordinates of nodes
        % Loop over all integration points
        for j=1:npts_per_gcell
            N = bfun(gcells,pc(j,:));
            Nder = bfundpar(gcells,pc(j,:));
            J = Jacobian_matrix(gcells,Nder,x);
            Jac = Jacobian_volume(gcells,conn, N, J, x);
            xyz = N'*x;
            b = get_magn(bl,xyz,J);
            Fe{i} = Fe{i} + reshape(b*N',dim*nfens,1) * Jac * weights(j);
        end
    end
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
end
