% Compute the  resistive heating source-term load vectors of the individual gcells.
%
% function evs = resistive_heating_loads(self, geom, temp, phi)
%
% Return an array of them so they may be assembled.
%    Call as
%       evs = resistive_heating_loads(feb, geom, temp, phi)
%    where
%       geom=geometry field
%       temp=temperature field
%       phi=Electric potential field
%
% Note: The electrical conductivity may depend on the temperature.
function evs = resistive_heating_loads(self, geom, temp, phi)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Precompute basis f. values + basis f. gradients wrt parametric coor
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material
    mat = get(self.feblock, 'mater');
    sigma = get(get(mat,'property'),'electrical_conductivity');
    if (strcmp(class(sigma),'function_handle'))
        need_to_eval=1;
    else
        need_to_eval=0; Sigma = sigma;
    end
     % Prepare some data: connectivity and the element matrices
    conns = get(gcells, 'conn'); % connectivity
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(temp,conns(i,:),'eqnums');
        Fe{i} =zeros(nfens,1);
    end
    xs =get(geom,'values');% retrieve the geometry information
    for i=1:size(conns,1)
        conn =conns(i,:);
        x=xs(conn,:);
        V = gather(phi, conn, 'values', 'noreshape');
        T = gather(temp,conn,'values','noreshape');
        % Loop over all integration points
        for j=1:npts_per_gcell
            J = Jacobian_matrix(gcells,Nders{j},x);
            Jac = Jacobian_volume(gcells,conn, Ns{j}, J, x);
            Ndersp = Nders{j}/J;
            gradphi = V'*Ndersp;
            if (need_to_eval)
                Sigma = sigma (Ns{j}'*T);
            end
            Fe{i} = Fe{i} + Ns{j} * ((gradphi*Sigma*gradphi') * Jac * w(j));
        end
    end
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
end
