% Number the equations.
%
% function retobj = numbereqns (self,varargin)
%
% Call as for field f:
%      f=numbereqns(f);
% or
%      f=numbereqns(f,n);
% where
% n=either (1) a scalar N which is the total number of equations; the 
%          numbering of the remaining degrees of freedom will start at N+1;
%       or (2) an array of equation numbers when size(n)==[nfens,dim];
%       or (3) permutation vector of length self.neqns to be applied 
%          to the current numbering;
%       or (4) cell array of linkages of nodes; each cell holds a structure
%          LINK such that LINK.slave is a list of nodes whose LINK.component
%          is constrained to be the same as the  LINK.component of the master 
%          node LINK.master;
% 
% Note that we're assigning to f: f changes inside this function!
%
% The free components in the field are numbered consecutively.
% No effort is made to optimize the numbering in any way. If you'd like to 
% optimize the numbering of the equations use the above form that sets
%
function retobj = numbereqns (self,varargin)
    invalid_eqnum = get(sysmat, 'invalid_eqnum');
    [nfens,dim] = size(self.values);
    arg= [];
    if nargin > 1
        arg=varargin{1};
    end
    links=[];
    if ~isempty(arg)
        if (prod(size(arg))==1) && (~iscell(arg))
            % the current number of equations is being modified 
            % for the subsequent numbering
            self.neqns = arg;
        elseif (size(arg)==[nfens,dim])
            %the equation numbers are being supplied in an array
            self.eqnums=arg;
            self.neqns =max(max(self.eqnums));
            retobj = self;
            return;
        elseif (length(arg)==self.neqns)
            %a permutation vector is supplied
            permutation=arg;
            k=1;
            for i=1:nfens
                for j=1:dim
                    if (~self.is_prescribed(i,j))
                        self.eqnums(i,j) = permutation(k);
                        k=k+1;
                    end
                end
            end
            retobj = self;
            return
        elseif iscell(arg)
            %cell array defining linkages between nodes is supplied
            links=zeros(size(self.eqnums));
            for i=1:length(arg )
                link=arg{i};
                for j=1:length( link.component )
                    for k=1:link.slave
                        M=min([link.slave(k), link.master]);
                        S=max([link.slave(k), link.master]);
                        links(S,link.component(j))=M;
                    end
                end
            end 
        else
            error('Argument purpose not recognized');
        end
    else
        self.neqns = 0;
    end
    % regular numbering
    for i=1:nfens
        for j=1:dim
            if (~self.is_prescribed(i,j))% free degree of freedom
                if (~ isempty(links))
                    if (links(i,j)~=0)% is this a slave?
                        self.eqnums(i,j) = self.eqnums(links(i,j),j);
                    else
                        self.neqns = self.neqns + 1;
                        self.eqnums(i,j) = self.neqns;
                    end
                else
                    self.neqns = self.neqns + 1;
                    self.eqnums(i,j) = self.neqns;
                end
            else% prescribed degree of freedom: no equation
                self.eqnums(i,j) = invalid_eqnum;
            end
        end
    end
    retobj = self;
end


