% Update material state.  
%
% function [out, newms] = update (self, ms, context)
%
% Update material state.  Return the updated material state, and the
% requested quantity (default is the heat flux).
%   Call as:
%     [out,newms] = update(m, ms, context)
%  where
%     m=material
%     ms = material state
%     context=structure
%        with optional fields
%           gradtheta= temperature gradient
%        with optional fields
%           gradphi= Voltage gradient
%        and optional fields
%           output=type of quantity to output, and interpreted by the
%           particular material; [] is returned when the material does not
%           recognize the requested quantity to indicate uninitialized
%           value.  It can be tested with isempty ().
%              output ='heat_flux' - heat flux vector; this is the default
%                      when output type is not specified.
%              output ='electrical_flux' - Electrical flux vector
%   The output arguments are
%     out=requested quantity
%     newms=new material state; don't forget that if the update is final
%           the material state newms must be assigned and stored.  Otherwise
%           the material update is lost!
%
function [out, newms] = update (self, ms, context)
    if isfield(context,'output')
        switch context.output
            case 'heat_flux'
                Kappa = get(self.thermal_property,'thermal_conductivity');
                gradtheta=reshape (context.gradtheta, length (context.gradtheta),1);
                flux = - Kappa * gradtheta;
                out = flux;
            case 'electrical_flux'
                sigma = get(self.electrical_property,'electrical_conductivity');
                if (strcmp(class(sigma),'function_handle'))
                    Sigma = sigma (context.T);
                else
                    Sigma = sigma;
                end
                gradphi=reshape (context.gradphi, length (context.gradphi),1);
                flux = - Sigma * gradphi;
                out = flux;
            otherwise
                out = [];
        end
    else
        out = [];
    end
    newms = ms;
    return;
end
