%%  Argyris L-frame, modal analysis.
% 
% Modal analysis of the L-frame of Argyris.

function b
    %%
    % Parameters:
    E=71240.0;%MPa
    nu=0.31;% Poisson ratio
    rho=5e-9;
    b=3; h=30; L=240;% cross-sectional dimensions and length of each leg in millimeters
    magn=40;% Magnitude of the total applied force, Newton
    utol=1e-3;
    neigvs=2;
    
    
    %%
    % Mesh
    n=8;
    tolerance=L/n/100;
    Parameters=Beam_cross_section(struct('type','rectangle','d2',b,'d3',h));
    Parameters.x1x2_vector=[0 1 0];
    [fens1,gcells1] = Beam_member([0,0,L;L,0,L],n,@gcellset_beam3,Parameters);
    [fens2,gcells2] = Beam_member([L,0,L;L,0,0],n,@gcellset_beam3,Parameters);
    [fens,gcells1,gcells2] = merge_meshes(fens1, gcells1, fens2, gcells2, tolerance);
    gcells= cat(gcells1,gcells2);
    
    % The clamped end is
    clampedn=fenode_select(fens, struct('box', update_box([],[0,0,L]), 'inflate', tolerance));
    % The force is applied at the tip
    tipn=fenode_select(fens, struct('box', update_box([],[L,0,0]), 'inflate', tolerance));
    
    %%
    % Properties
    prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
    mater = mater_defor_ss_linel_triax (struct('property',prop));
    
    %%
    % Finite element block
    feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells));
    
    %%
    % Initialize configuration variables
    geom0 = field(struct ('name',['geom0'], 'dim', 3, 'fens',fens));
    u0=0*geom0;
    Rfield0 = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom0,'nfens'),1)*reshape(eye(3),1,9)));
    % Define the displacement increment field
    dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom0,'nfens'),6)));
    
    %%
    % Apply EBC's
    ebc_fenids=[clampedn];
    ebc_prescribed=[1];
    ebc_comp=[];
    ebc_val=ebc_fenids*0;
    dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    dchi  = apply_ebc (dchi);
    
    %%
    % Number equations
    dchi  = numbereqns (dchi);
    neqns=get(dchi, 'neqns');
    
    gv=graphic_viewer;
    
    K = start (sparse_sysmat, neqns);
    K = assemble (K, stiffness(feb,geom0,u0,Rfield0,dchi));
    M = start (sparse_sysmat, neqns);
    M = assemble (M, mass(feb,geom0,u0,Rfield0,dchi));
    Km=get(K, 'mat');
    Mm=get(M, 'mat');
    
    [W,Omega]=eigs(Km,Mm, neigvs,'sm');
    [Omegas2,ix]=sort((diag(Omega))) ;
    Frequencies= sqrt(Omegas2)/2/pi
    for i=1:neigvs
        disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(Frequencies(i)) '  [Hz]']);
    end
    
    gv=graphic_viewer;
    gv=reset (gv,[]);
    scale=0.50;
    for i=1:neigvs
        dchi = scatter_sysvec(dchi, W(:,ix(i)));
        transl=slice(dchi,(1:3),'transl');
        rotat=slice(dchi,(4:6),'rotat');
        wv = get(transl,'values');
        wmag = sqrt(wv(:,1).^2+wv(:,2).^2+wv(:,3).^2);
        dcm=data_colormap(struct ('range',[min(wmag),max(wmag)], 'colormap',jet));
        colors=map_data(dcm, wmag);
        colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
        for xscale=scale*sin((0:1:42)/21*2*pi)
            reset (gv,[]);
            title(['  Eigenvector ' num2str(i) ', frequency ' num2str(Frequencies(i)) ]);
            camset(gv,1.0e+003 *[-1.2286   -1.7890    1.4418    0.1411   -0.0040    0.1428         0         0    0.0010    0.0086]);
            draw(feb, gv, struct ('x', geom0, 'u', 0*transl, 'rot', 0*rotat, 'facecolor','none'));
            draw(feb, gv, struct ('x', geom0, 'u', xscale*transl, 'rot', xscale*rotat,'colorfield',colorfield));
            draw_axes (gv,struct('length',L/10 ));
            pause(0.005);
        end
        pause(2); % next eigenvector
    end
    
    
  assignin('caller','faesor_test_passed',((norm([11.2732;30.5269]-Frequencies(1:2)))<1e-3));
end