%%  Transverse vibration of a pre-bent beam under harmonic compressive force
% 
% A slightly curved beam is subject to harmonic compressive force
% (static component plus a dynamic one). This leads to classical
% development of instabilities for certain combinations of forces (Mathieu
% equation).
function CB_mathieu
    %%
    % Parameters:
    E=71240.0;%MPa
    nu=0.31;% Poisson ratio
    rho=2.7e-9;
    Width=6; Height=60; Length=1300;% cross-sectional dimensions and length of each leg in millimeters
    Parameters=Beam_cross_section(struct('type','rectangle','d2',Width,'d3',Height));
    Imperfection=Width/10;
    g=9.81e3; % 9.81 meters per second squared
    
    %%
    % Natural frequency of free vibration, without the action of pre-stress
    analyt_frequency_1=(1*pi)^2/(2*pi*Length^2)*sqrt(E*Parameters.I3/rho/Parameters.A);
    dt=1/analyt_frequency_1/20;
    omega_1=2*pi*analyt_frequency_1;
 
    %%
    % Static buckling load
    P_Euler=(pi^2*E*Parameters.I3/Length^2);
    
    %%
    % 1. This set of parameters should produce violent oscillations
       p=0;% Static component of the  compressive force
       s=-1/13;
        Omega=omega_1*2;% frequency of excitation
        a=(omega_1/Omega)^2*(1-p)
        b=-(omega_1/Omega)^2*s
        tend=1361*dt; 
    
    %     %%
    %     % 2. This set of parameters should not produce excessive oscillations: the dynamic force is mis-tuned
    %         p=0;% Static component of the  compressive force
    %        s=-1/13;
    %         Omega=omega_1*2*1.05;% frequency of excitation
    %         a=(omega_1/Omega)^2*(1-p)
    %         b=-(omega_1/Omega)^2*s
    %         tend=1361*dt; 
    
    
    %%
    %         % 3. This set of parameters should not produce violent oscillations
        p=1/2;% Static component of the  compressive force
        s=-1.0;
        Omega=omega_1*sqrt(2/3 );% frequency of excitation
        a=(omega_1/Omega)^2*(1-p)
        b=-(omega_1/Omega)^2*s
        tend=1361*dt; 
    
    Newmarkg = 1/2; Newmarkb = 1/4*(1/2+Newmarkg)^2;
    % Newmarkg = 0.6; Newmarkb = 1/4*(1/2+Newmarkg)^2;
    maxit=30;
    graphics=0;
    camera=[-3.6172e+001 -5.8778e+001  1.6854e+003  4.3681e+000  1.3253e+000  6.4861e+002  5.5783e-001  8.2702e-001  6.9756e-002  8.1540e+000];
    Report =@(x) (disp(x) );
    
    % Choose the mass formulation:
    consistent_no_rotation_inertia=0;
    consistent_with_rotation_inertia=1;
    lumped_diagonal_no_rotation_inertia=2;
    lumped_diagonal_with_rotation_inertia=3;
    mass_type=1;
    
    %%
    % Mesh
    n=3;
    tolerance=Length/n/100;
    Parameters.x1x2_vector=[1 0 0];
    X=[0,0,0;Imperfection,0,Length/2;0,0,Length];
    [fens,gcells] = Beam_member(X,2*n,@gcellset_beam3,Parameters);
    
    % The supported end is
    pinn=fenode_select(fens, struct('box', update_box([],X(1,:)), 'inflate', tolerance));
    % The Tip
    rollern=fenode_select(fens, struct('box', update_box([],X(3,:)), 'inflate', tolerance));
    % The midpoint
    midn=fenode_select(fens, struct('box', update_box([],X(2,:)), 'inflate', tolerance));
    
    %%
    % Properties
    prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
    mater = mater_defor_ss_linel_triax (struct('property',prop));
    
    %%
    % Finite element block
    feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells,...
        'mass_type',mass_type));
    
    %%
    % Initialize configuration variables
    geom0 = field(struct ('name',['geom0'], 'dim', 3, 'fens',fens));
    u=0*geom0;
    Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom0,'nfens'),1)*reshape(eye(3),1,9)));
    Rfield0=Rfield;
    % Define the displacement increment field
    dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom0,'nfens'),6)));
    
    % Apply EBC's
    ebc_fenids=[pinn];
    ebc_prescribed=[1];
    ebc_comp=[1,2,3,6];
    ebc_val=0;
    dchi = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    ebc_fenids=[rollern];
    ebc_prescribed=[1];
    ebc_comp=[1,2,6];
    ebc_val=0;
    dchi = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    dchi = apply_ebc (dchi);
    % Number equations
    dchi = numbereqns (dchi);
    neqns=get(dchi, 'neqns');
    
    
    if (graphics)
        gv=graphic_viewer;
    else
        hold on;
        axis equal
        ;
    end
    
    % Now comes the nonlinear solution
    dchi = dchi*0; % zero out the displacement
    v = dchi*0; % zero out the Velocity
    xyz = gather(geom0,(1:count(fens)),'values','noreshape');
    a = dchi*0; % zero out the Acceleration
    utol = 1e-13*get(dchi, 'neqns');
    
    K = start (sparse_sysmat, get(dchi, 'neqns'));
    M = start (sparse_sysmat, get(dchi, 'neqns'));
    G = start (sparse_sysmat, get(dchi, 'neqns'));
    
    Mid_ux= [];
    t=0; %
    step=0;
    while (t <= tend)
        t = t + dt;
        Report(['Time ' num2str(t) ]); % pause
        % Initialization
        dchi = apply_ebc(dchi);% Apply boundary conditions
        u1 = u; % guess
        Rfield1=Rfield; % guess
        stepdchi = 0*apply_ebc(dchi);% Total increment in current step
        a1 = -(1/Newmarkb/dt)*v -(1/2-Newmarkb)/Newmarkb*a;
        v1 = v + dt*((1-Newmarkg)*a + Newmarkg*a1);
        dchipv =gather_sysvec(dt*v +(dt^2/2*(1-2*Newmarkb))*a);
        vpv =gather_sysvec(v +(dt*(1-Newmarkg))*a);
        nl=nodal_load(struct ('id',rollern,'dir',[3],'magn',[cos(Omega*t)*s+p]*P_Euler));
        
        iter=1;
        while 1
            feb1=feb;
            F = start (sysvec, get(dchi, 'neqns'));
            F = assemble (F, loads(nl, dchi));
            [feb,evs] = restoring_force(feb,geom0,u1,Rfield1,dchi);       % Internal forces
            F = assemble (F, evs);
            %             K =assemble (K,cat(2,stiffness(feb,geom0,u1,Rfield1,dchi),stiffness_geo(feb,geom0,u1,Rfield1,dchi)));
            K =assemble (K,cat(2,stiffness(feb,geom0,u1,Rfield1,dchi)));
            Km=get(K, 'mat');
            M =assemble (M,mass(feb,geom0,u1,Rfield1,dchi));
            Mm=get(M, 'mat');
            G =assemble (G,coriolis(feb,geom0,u1,Rfield1,v1,dchi));
            Gm=get(G, 'mat');
            rhs=get(F,'vec')...
                +Mm*((-1/(Newmarkb*dt^2))*gather_sysvec(stepdchi)+(1/(Newmarkb*dt^2))*dchipv)...
                +Gm*((-Newmarkg/Newmarkb/dt)*gather_sysvec(stepdchi)+(Newmarkg/Newmarkb/dt)*dchipv-vpv);
            dchi = scatter_sysvec(dchi, (Km+(Newmarkg/Newmarkb/dt)*Gm+(1/(Newmarkb*dt^2))*Mm)\rhs); % Displacement increment
            u1 = u1 + slice(dchi,(1:3),'transl');   % increment displacement
            stepdchi = stepdchi + dchi;
            v1 = v1 + (Newmarkg/Newmarkb/dt)*dchi;
            a1 = a1 + (1/Newmarkb/dt^2)*dchi;
            Rfield1=update_rotation_field(Rfield1,slice(dchi,(4:6),'rotat'));
            %             Report(['   It. ' num2str(iter) ': ||du||=' num2str(norm(dchi))]);
            if (max(abs(gather(dchi,[],'values'))) < utol) break; end;% convergence check
            if (iter >maxit)% bailout for failed convergence
                error( [' Possible failed convergence']);
            end
            K = reinitialize (K);
            M = reinitialize (M);
            G = reinitialize (G);
            iter=iter+1;
        end
        [feb,ignore] = restoring_force(feb,geom0,u1,Rfield1,dchi);        % final update
        %         Report(['    Converged for t=' num2str(t)]); % pause
        u = u1;       % update the displacement
        Rfield=Rfield1; % update the rotations
        v = v1;       % update the velocities
        a = a1;       % update the accelerations
        
        if graphics
            gv=reset(gv,[]);
            draw(feb, gv, struct ('x', geom0, 'u', 0*u1, 'R', Rfield0, 'facecolor','none'));
            draw(feb, gv, struct ('x', geom0, 'u', u1, 'R', Rfield1, 'facecolor','g'));
            %         draw_local_coordinates(feb, gv, struct ('x', geom0, 'u', u1, 'R', Rfield1, 'length',12.5));
            camset(gv, camera);
            drawnow;
        else
            midu=gather(u1,midn, 'values', 'noreshape');
            Mid_ux=[Mid_ux;midu(1)];
             skip=20;
            if (~isempty(Mid_ux)) && (mod(size(Mid_ux,1),skip)==0)
                if (size(Mid_ux,1)> skip), iskip=skip; else; iskip=skip-1; end
                plot((t-(iskip:-1:0)*dt),Mid_ux(end-iskip:end,1),'k-','linewidth',3); hold on
                xlabel('Time');
                ylabel('Midpoint transverse displacement');
                                set_graphics_defaults 
                set(gca,'xlimmode','manual');
                set(gca,'ylimmode','manual');
                set(gca,'DataAspectRatioMode','auto');
                set(gca,'xlim', [0,tend]);
                set(gca,'ylim', 1.2*[-10,+10]);
                grid on
                drawnow;figure(gcf);
            end
           
            
        end
        step=step+1;
    end
end
