% Automatic triangulation of a general 2-D domain. 
%
% function [fens,gcells,groups,edge_gcells,edge_groups] =
%                     targe2_mesher(fs,thickness,varargin)
%
% Automatic triangulation of a general 2-D domain. The input is given as a cell array
% of commands to the mesher Targe2. These are described in the manual,
% targe2_users_guide.pdf.
% Input:
% fs -- cell array of strings, each string a command recognized
%       by the mesher (targe2_users_guide.pdf) 
% thickness -- thickness of the two-dimensional slab
% varargin-- optional argument: structure with name-value pairs
%                - quadratic: convert the triangulation to
%                             quadratic elements, true or false;
%                - merge_tolerance: see this option in
%                             targe2_users_guide.pdf
%                - and any other attributes recognized by the constructors
%                of the geometric cells, for instance axisymm.
% Output:
%  fens= the finite element nodes
%  gcells=the triangles
% The following are optional outputs:
%  groups=cell array of indexes of the triangles in the individual subregions;
%  For instance groups{3} is the list of the triangles that belong to subregion 3
%  edge_gcells=cell array of the edge elements
%  edge_groups=cell array of indexes of the edge elements on the individual edges;
%
function [fens,gcells,groups,edge_gcells,edge_groups] = targe2_mesher(fs,thickness,varargin)
    in='in';
    out ='out';
    quadratic= false;
    
    if nargin >2
        options=varargin{1};
    end
    options.id=0;
    options.conn = [];
    if isfield(options,'axisymm')
        if ~options.axisymm
            options.other_dimension=thickness;
        end
    else
        options.other_dimension=thickness;
    end
    merge_tolerance=eps;
    if isfield(options,'merge_tolerance')
        merge_tolerance =options.merge_tolerance;
    else
        merge_tolerance =eps;
    end

    if nargin >2
        if isfield(options,'quadratic')
            quadratic = options.quadratic;
        end
    end

    d=pwd;
    if ~isdir(FAESOR_work_path)
        mkdir(FAESOR_work_path);
    end
    cd(FAESOR_work_path);

    fid =fopen ([in], 'w');
    for i=1:length(fs)
        fprintf(fid,'%s\n',fs{i});
    end
    fclose (fid);

    if strcmp(computer,'PCWIN')
%         c=['"' finesse_path filesep 'meshing' filesep 'targe2' filesep 'Targe2.exe" -m ' num2str(options.merge_tolerance) ' -i ' in ' -f  2  -o ' out];
        c=['"' FAESOR_path filesep 'meshing' filesep 'targe2' filesep 'Targe2.exe" ' ' -i ' in ' -f  2  -o ' out];
    elseif strcmp(computer,'GLNX86')  
        exec=['"' FAESOR_path filesep 'meshing' filesep 'targe2' filesep 'targe2_GLNX86"'];
        c=[exec ' -i ' in ' -f  2 -o ' out ];
        system(['chmod +x ' exec]);
    elseif strcmp(computer,'GLNXA64')
        exec=['"' sofea_path filesep 'meshing' filesep 'targe2' filesep 'targe2_GLNXA64"'];
        c=[exec ' ' use_quadtree ' ' ' -i ' in ' -f  2 -o ' out ];
        system(['chmod +x ' exec]);
    elseif strcmp(computer,'MACI')
        exec=['"' FAESOR_path filesep 'meshing' filesep 'targe2' filesep 'targe2_MACI"'];
        c=[exec ' -i ' in ' -f  2 -o ' out ];
        system(['chmod +x ' exec]);
    elseif strcmp(computer,'PCWIN64')
%         c=['"' finesse_path filesep 'meshing' filesep 'targe2' filesep 'Targe2.exe" -m ' num2str(options.merge_tolerance) ' -i ' in ' -f  2  -o ' out];
        c=['"' FAESOR_path filesep 'meshing' filesep 'targe2' filesep 'Targe2.exe" ' ' -i ' in ' -f  2  -o ' out];
    else
        error(['Computer platform ' computer ' is not supported: contact the author'])
    end
    system (c);

    fid =fopen ([out], 'r');
    l=fgets(fid);
    totals =sscanf(l,'%d');
    xy =fscanf(fid,'%g', [4,totals(1)]);
    p = xy';

    es =fscanf(fid,'%g', [4,totals(2)]);
    es = es';

    ts =fscanf(fid,'%g', [5,totals(3)]);
    ts = ts';

    fclose (fid);

    %     Assign groups
    for i= 1:totals(3)
        group =ts(i,5);
        if ~exist('groups')
            groups{group} = [];
        end
        if length(groups)<group
            groups{group} = [];
        end
        groups{group} = [groups{group} i];
    end
    for i= 1:totals(2)
        group =es(i,4);
        if group~=0
            if ~exist('edge_groups')
                edge_groups{group} = [];
            end
            if length(edge_groups)<group
                edge_groups{group} = [];
            end
            edge_groups{group} = [edge_groups{group} i];
        end
    end

    nnodes= size (p, 1);
    fens=fenodeset(struct('id',p(:,1),'xyz',p(:,2:3)));
    
    if quadratic
        nedges=size(es, 1);
        nmidnodes =nedges;
        es= [es, (nnodes+1:nnodes+nmidnodes)'];
        xy=get(fens,'xyz');
        id=get(fens,'id');
        xy(nnodes+1:nnodes+nmidnodes,:)=zeros(nmidnodes,2);
       id(nnodes+1:nnodes+nmidnodes)=0;
        edge_conns=zeros(nedges,3);
        for i=1:nedges
            midx=p(es(i,2:3),2:3);
            id(nnodes+i)=nnodes+i;
            xy(nnodes+i,:)=(midx(1,:) +midx(2,:))/2;
            edge_conns(i,:) =[es(i,2:3),nnodes+i];
        end
        options.conn=edge_conns;
        edge_gcells=gcellset_L3(options);
        fens=fenodeset(struct('id',id,'xyz',xy));
        
        ncells=size (ts, 1);
        conns=zeros(ncells,6);
        for i=1:ncells
            nns=[intersect(es(ts(i,4),2:3),es(ts(i,2),2:3)),...
                intersect(es(ts(i,2),2:3),es(ts(i,3),2:3)),...
                intersect(es(ts(i,3),2:3),es(ts(i,4),2:3))];
            enns=es(ts(i,2:4),5)';
            nns = [nns, enns];
            if det([1 p(nns(1),2:3); 1 p(nns(2),2:3); 1 p(nns(3),2:3)])<0
                nns=nns([1, 3, 2, 6, 5, 4]);
            end
            conns(i,:) =nns;
        end
        options.conn=conns;
        gcells=gcellset_T6(options);
    else
        ncells=size (ts, 1);
        options.id =zeros(ncells,1);
        options.conn =zeros(ncells,3);
        for i=1:ncells
            nns=unique([es(ts(i,2),2:3) es(ts(i,3),2:3) es(ts(i,4),2:3)]);
            if det([1 p(nns(1),2:3); 1 p(nns(2),2:3); 1 p(nns(3),2:3)])<0
                nns =nns([1, 3, 2]);
            end
            options.id(i) =i;
            options.conn(i,:) =nns;
        end
        gcells=gcellset_T3(options);
        
        nedges=size(es, 1);
        options.id =zeros(nedges,1);
        options.conn =zeros(nedges,2);
        options.other_dimension=1.0;
        for i=1:nedges
            options.id(i) =i;
            options.conn(i,:) =es(i,2:3);
        end
        edge_gcells=gcellset_L2(options);
    end
    cd(d);

    % mesh{1}=fens;
    % mesh{2}=gcells;
    % drawmesh(mesh); view (2);

    return;
end

